#coding: utf-8
import json
import os
import sys
import threading
import time

import requests
import wx
from pubsub import pub

from ui import globalParams


class EventAndOtherArea():
    def __init__(self, frame):
        self.frame = frame
        self.mainPanel = None
        self.sendEvtUrl = "http://api-test.vandyo.com/mirror/event"      # 事件发送地址

        self.sendEvtUrlTextCtr = None                             # 发送事件消息地址框
        self.accStatusCombox = None                               # 点火状态设置框
        self.acrTextCtr = None                                    # 定位精度设置框
        self.altTextCtr = None                                    # 海拔设置框
        self.ctpTextCtr = None                                    # 坐标系类型设置框
        self.drtTextCtr = None                                    # 方向角设置框
        self.gacrComboBox = None                                  # 卫星信号强度设置框
        self.gtmTextCtr = None                                    # 时间设置框
        self.latTextCtr = None                                    # 维度设置框
        self.lngTextCtr = None                                    # 经度设置框
        self.ltpTextCtr = None                                    # 定位结果来源设置框
        self.satlTextCtr = None                                   # 定位星数设置框
        self.scnlTextCtr = None                                   # 室内外置信度设置框
        self.spdlTextCtr = None                                   # 速度设置框
        self.trsComboBox = None                                   # 定位可信度设置框

        self.videoFilePathTextCtr = None                          # 上传文化底子框
        self.cameraTypeCombox = None                              # 摄像头选择框
        self.videoTypeCombox = None                               # 上传文件类型选择框

        self.logTextCtr = None                                    # 日志输出框


    def getLatTextCtr(self):
        return self.latTextCtr
    def getLngTextCtr(self):
        return self.lngTextCtr

    #################################################
    #       创建一个顶级pannel
    #################################################
    def create(self):
        self.mainPanel = wx.Panel(self.frame)
        boxSizer = wx.BoxSizer(wx.VERTICAL)
        topPanel = self.getTopPanel()
        bottomPanel = self.getBottomPanel()
        boxSizer.Add(topPanel,5,flag=wx.EXPAND | wx.ALL)
        boxSizer.Add(bottomPanel,1, flag=wx.EXPAND | wx.TOP , border=5)
        self.mainPanel.SetSizer(boxSizer)

        pub.subscribe(self.showLog, "showLog")
        return self.mainPanel

    #################################################
    #       获取顶部pannel的内容
    #################################################
    def getTopPanel(self):
        topPanel = wx.Panel(self.mainPanel)
        topPanelSizer = wx.BoxSizer(wx.VERTICAL)
        basicInfoPanel = wx.Panel(topPanel, -1, size=(wx.EXPAND, 40),style=wx.BORDER_SIMPLE)
        wx.StaticText(basicInfoPanel, label='发送地址：', pos=(10, 10))
        self.sendEvtUrlTextCtr = wx.TextCtrl(basicInfoPanel, pos=(80, 5), size=wx.Size(350, -1), value=self.sendEvtUrl)
        gpsPannel = wx.StaticBox(topPanel, -1, "gps数据：", size=(wx.EXPAND, 120))
        wx.StaticText(gpsPannel, label='点熄火状态：',pos=(10, 20))
        self.accStatusCombox = wx.ComboBox(gpsPannel, pos=(90, 15), choices=['点火', '熄火'], value="点火",
                                              size=wx.Size(60, -1))
        wx.StaticText(gpsPannel, label='定位精度：', pos=(160, 20))
        self.acrTextCtr = wx.TextCtrl(gpsPannel, pos=(230, 15), size=wx.Size(70, -1), value="98.88")
        wx.StaticText(gpsPannel, label='海拔（米）：', pos=(310, 20))
        self.altTextCtr = wx.TextCtrl(gpsPannel, pos=(390, 15), size=wx.Size(70, -1), value="100.0")
        wx.StaticText(gpsPannel, label='坐标系类型：', pos=(470, 20))
        self.ctpTextCtr = wx.TextCtrl(gpsPannel, pos=(550, 15), size=wx.Size(70, -1), value="1")
        wx.StaticText(gpsPannel, label='方向角：', pos=(630, 20))
        self.drtTextCtr = wx.TextCtrl(gpsPannel, pos=(690, 15), size=wx.Size(70, -1), value="48.5")
        wx.StaticText(gpsPannel, label='卫星信号强度：', pos=(770, 20))
        self.gacrComboBox = wx.ComboBox(gpsPannel, pos=(860, 15), choices=['好', '差'], value="好",
                                              size=wx.Size(60, -1))
        wx.StaticText(gpsPannel, label='时间：', pos=(10, 55))
        times = self.getCurTime()
        self.gtmTextCtr = wx.TextCtrl(gpsPannel, pos=(60, 50), size=wx.Size(130, -1), value=times)
        wx.StaticText(gpsPannel, label='纬度：', pos=(200, 55))
        self.latTextCtr = wx.TextCtrl(gpsPannel, pos=(240, 50), size=wx.Size(100, -1), value="29.569187")
        wx.StaticText(gpsPannel, label='经度：', pos=(350, 55))
        self.lngTextCtr = wx.TextCtrl(gpsPannel, pos=(390, 50), size=wx.Size(100, -1), value="106.592613")
        wx.StaticText(gpsPannel, label='定位结果来源：', pos=(500, 55))
        self.ltpTextCtr = wx.TextCtrl(gpsPannel, pos=(590, 50), size=wx.Size(80, -1), value="1")
        wx.StaticText(gpsPannel, label='定位星数：', pos=(680, 55))
        self.satlTextCtr = wx.TextCtrl(gpsPannel, pos=(760, 50), size=wx.Size(80, -1), value="2")
        wx.StaticText(gpsPannel, label='室内外置信度：', pos=(10, 90))
        self.scnlTextCtr = wx.TextCtrl(gpsPannel, pos=(100, 85), size=wx.Size(80, -1), value="89")
        wx.StaticText(gpsPannel, label='速度：', pos=(190, 90))
        self.spdlTextCtr = wx.TextCtrl(gpsPannel, pos=(230, 85), size=wx.Size(80, -1), value="60.8")
        wx.StaticText(gpsPannel, label='定位可信度：', pos=(320, 90))
        self.trsComboBox = wx.ComboBox(gpsPannel, pos=(400, 85), choices=['非常可信', '可信度一般','可信度较低','非常不可信'], value="可信度一般",
                                        size=wx.Size(100, -1))

        ctrlPannel = wx.StaticBox(topPanel, -1, "控制：", size=(wx.EXPAND, 50))
        disDrivingCrashButton = wx.Button(ctrlPannel, label="停车碰撞", pos=(10, 20))
        self.frame.Bind(wx.EVT_BUTTON, lambda evt: self.sendDisDrivingCrashEvt(evt),
                        disDrivingCrashButton)
        drivingCrashButton = wx.Button(ctrlPannel, label="行车碰撞", pos=(90, 20))
        self.frame.Bind(wx.EVT_BUTTON, lambda evt: self.sendDrivingCrashEvt(evt),
                        drivingCrashButton)
        helpButton = wx.Button(ctrlPannel, label="一键救援", pos=(170, 20))
        self.frame.Bind(wx.EVT_BUTTON, lambda evt: self.sendHelpEvt(evt),
                        helpButton)

        videoUploadPannel = wx.StaticBox(topPanel, -1, "视频上传：", size=(wx.EXPAND, 50))
        wx.StaticText(videoUploadPannel, label='文件：', pos=(10, 20))
        self.videoFilePathTextCtr = wx.TextCtrl(videoUploadPannel, pos=(60, 15), size=wx.Size(350, -1))
        selectFileButton = wx.Button(videoUploadPannel, label="选择文件", pos=(420, 15))
        self.frame.Bind(wx.EVT_BUTTON, lambda evt: self.selectVideoFile(evt,self.videoFilePathTextCtr),
                        selectFileButton)
        wx.StaticText(videoUploadPannel, label='视频类型：', pos=(500, 20))
        self.videoTypeCombox = wx.ComboBox(videoUploadPannel, pos=(570, 15), choices=['手动抓拍图片', '手动抓拍视频',"历史视频", \
                                        "行车碰撞视频","停车碰撞视频","一键报警视频"], value="手动抓拍图片",size=wx.Size(100, -1))
        self.cameraTypeCombox = wx.ComboBox(videoUploadPannel, pos=(680, 15), choices=['摄像头1', '摄像头2', "摄像头3", ],value="摄像头1",size=wx.Size(80, -1))
        startUploadButton = wx.Button(videoUploadPannel, label="上传文件", pos=(770, 15))
        self.frame.Bind(wx.EVT_BUTTON, lambda evt: self.uploadVideoFile(evt),
                        startUploadButton)
        topPanelSizer.Add(basicInfoPanel, 0.7, flag=wx.EXPAND | wx.ALL)
        topPanelSizer.Add(gpsPannel, 2, flag=wx.EXPAND | wx.ALL)
        topPanelSizer.Add(ctrlPannel, 1, flag=wx.EXPAND | wx.ALL)
        topPanelSizer.Add(videoUploadPannel, 1, flag=wx.EXPAND | wx.TOP,border=10)
        otherPannel = wx.Panel(topPanel, -1, size=(wx.EXPAND, 50))
        topPanelSizer.Add(otherPannel, 3, flag=wx.EXPAND | wx.ALL)
        topPanel.SetSizer(topPanelSizer)

        return topPanel


    #################################################
    #       获取底部pannel的内容
    #################################################
    def getBottomPanel(self):
        bottomPanel = wx.Panel(self.mainPanel)
        boxSizer_bottom = wx.BoxSizer(wx.HORIZONTAL)
        logArea = wx.Panel(bottomPanel)
        logStaticText = wx.StaticText(logArea, label='日志：')
        self.logTextCtr = wx.TextCtrl(logArea, style=wx.TE_MULTILINE)
        boxSizer_logArea = wx.BoxSizer(wx.VERTICAL)
        boxSizer_logArea.Add(logStaticText, 2, flag=wx.EXPAND | wx.TOP ,border=10)
        boxSizer_logArea.Add(self.logTextCtr, 100, flag=wx.EXPAND | wx.ALL)
        logArea.SetSizer(boxSizer_logArea)

        ctrArea = wx.Panel(bottomPanel)
        ctrText = wx.StaticText(ctrArea, label='控制：')
        ctrPanel = wx.Panel(ctrArea, style=wx.BORDER_SIMPLE)
        clearLogButton = wx.Button(ctrPanel, label="清空日志", pos=(5, 5))
        self.frame.Bind(wx.EVT_BUTTON, lambda evt: self.clearLog(evt),
                        clearLogButton)
        boxSizer_ctrArea = wx.BoxSizer(wx.VERTICAL)
        boxSizer_ctrArea.Add(ctrText,2, flag=wx.EXPAND | wx.TOP ,border=10)
        boxSizer_ctrArea.Add(ctrPanel,100, flag=wx.EXPAND | wx.ALL)
        ctrArea.SetSizer(boxSizer_ctrArea)

        boxSizer_bottom.Add(logArea,70,flag=wx.EXPAND | wx.ALL)
        boxSizer_bottom.Add(ctrArea, 30, flag=wx.EXPAND | wx.LEFT,border=10)
        bottomPanel.SetSizer(boxSizer_bottom)

        return  bottomPanel

    ####################################################
    #      发送停车告警碰撞事件
    ####################################################
    def sendDisDrivingCrashEvt(self,evt):
        data = {}
        eventTime = int(time.time())
        eventType = 1
        imei = globalParams.homeArea.getDevCodeText().GetValue()
        data["eventTime"] = eventTime
        data["eventType"] = eventType
        data["gpsData"] = self.getGpsData()
        data["imei"] = imei

        url = self.sendEvtUrlTextCtr.GetValue()
        headers = {'Content-Type': 'application/json'}
        res = requests.post(url, headers=headers,data=json.dumps(data))
        curTime = self.getCurTimeTxt()
        self.logTextCtr.WriteText(curTime + res.text + "\n")

    ####################################################
    #      发送行车告警碰撞事件
    ####################################################
    def sendDrivingCrashEvt(self,evt):
        data = {}
        eventTime = int(time.time())
        eventType = 2
        imei = globalParams.homeArea.getDevCodeText().GetValue()
        data["eventTime"] = eventTime
        data["eventType"] = eventType
        data["gpsData"] = self.getGpsData()
        data["imei"] = imei

        url = self.sendEvtUrlTextCtr.GetValue()
        headers = {'Content-Type': 'application/json'}
        res = requests.post(url, headers=headers, data=json.dumps(data))
        curTime = self.getCurTimeTxt()
        self.logTextCtr.WriteText(curTime + res.text + "\n")


    ####################################################
    #      发送一键救援事件事件
    ####################################################
    def sendHelpEvt(self,evt):
        data = {}
        eventTime = int(time.time())
        eventType = 3
        imei = globalParams.homeArea.getDevCodeText().GetValue()
        data["eventTime"] = eventTime
        data["eventType"] = eventType
        data["gpsData"] = self.getGpsData()
        data["imei"] = imei

        url = self.sendEvtUrlTextCtr.GetValue()
        headers = {'Content-Type': 'application/json'}
        res = requests.post(url, headers=headers, data=json.dumps(data))
        curTime = self.getCurTimeTxt()
        self.logTextCtr.WriteText(curTime + res.text + "\n")

    ####################################################
    #      获取当前时间
    ####################################################
    def getCurTime(self):
        timeCur = int(time.time() * 1000)
        timeArray = time.localtime(timeCur / 1000)
        timeCur = time.strftime("%Y-%m-%d %H:%M:%S", timeArray)
        # timeCur = "[" + timeCur + "]  "
        return timeCur
    def getCurTimeTxt(self):
        timeCur = int(time.time() * 1000)
        timeArray = time.localtime(timeCur / 1000)
        timeCur = time.strftime("%Y-%m-%d %H:%M:%S", timeArray)
        timeCur = "[" + timeCur + "]  "
        return timeCur

    #################################################
    #       打开文件选择对话框
    #################################################
    def selectVideoFile(self,evt,textCtr):
        wildcard = "Video file (*.mp4;*.jpg)|*.mp4;*.jpg"
        dlg = wx.FileDialog(
            self.frame, message="选择flv视频文件",
            defaultDir=os.getcwd(),
            defaultFile="",
            wildcard=wildcard,
            style=wx.FD_OPEN |
                  wx.FD_CHANGE_DIR | wx.FD_FILE_MUST_EXIST |
                  wx.FD_PREVIEW
        )
        if dlg.ShowModal() == wx.ID_OK:
            path = dlg.GetPath()
            textCtr.SetValue(path)
        dlg.Destroy()


    #################################################
    #       清空日志
    #################################################
    def clearLog(self,evt):
        self.setLogRedirect()
        self.logTextCtr.SetValue("")

    ####################################################
    #      日志重定向到日志框
    ####################################################
    def setLogRedirect(self):
        if globalParams.isLogRedirect == 1:
            sys.stdout = self.logTextCtr
            sys.stderr = self.logTextCtr

    ####################################################
    #      获取gps数据
    ####################################################
    def getGpsData(self):
        data = {}
        acc = self.accStatusCombox.GetValue()
        accVal = 0
        if acc == "点火":
            accVal = 1
        acrVal = float(self.acrTextCtr.GetValue())
        altVal = float(self.altTextCtr.GetValue())
        ctpVal = int(self.ctpTextCtr.GetValue())
        drtVal = float(self.drtTextCtr.GetValue())
        gacr = self.gacrComboBox.GetValue()
        gacrVal = 0
        if gacr == "好":
            gacrVal = 1
        gtm = self.gtmTextCtr.GetValue()
        gtmVal = int(time.mktime(time.strptime(gtm, "%Y-%m-%d %H:%M:%S")))
        latVal = float(self.latTextCtr.GetValue())
        lngVal = float(self.lngTextCtr.GetValue())
        ltpVal = int(self.ltpTextCtr.GetValue())
        saltVal = int(self.satlTextCtr.GetValue())
        scnVal = int(self.scnlTextCtr.GetValue())
        spdVal = float(self.spdlTextCtr.GetValue())
        trs = self.trsComboBox.GetValue()
        trsVal = 0
        if trs == "非常可信":
            trsVal = 3
        elif trs == "可信度一般":
            trsVal = 2
        elif trs == "可信度较低":
            trsVal = 1

        data["acc"] = accVal
        data["acr"] = acrVal
        data["alt"] = altVal
        data["ctp"] = ctpVal
        data["drt"] = drtVal
        data["gacr"] = gacrVal
        data["gtm"] = gtmVal
        data["lat"] = latVal
        data["lng"] = lngVal
        data["ltp"] = ltpVal
        data["salt"] = saltVal
        data["scn"] = scnVal
        data["spd"] = spdVal
        data["trs"] = trsVal

        return data

    ####################################################
    #      上传视频文件
    ####################################################
    def uploadVideoFile(self,evt):
        threadObj = threading.Thread(target=self.doUploadVideoFile())
        threadObj.start()
        timeCur = self.getCurTimeTxt()
        self.logTextCtr.WriteText(timeCur + "文件开始上传！\n")

    def doUploadVideoFile(self):
        params = {}
        filePath = self.videoFilePathTextCtr.GetValue()
        camTxt = self.cameraTypeCombox.GetValue()
        cam = 1
        if camTxt == "摄像头1":
            cam = 1
        elif camTxt == "摄像头2":
            cam = 2
        elif camTxt == "摄像头3":
            cam = 3

        iscaptureTxt = self.videoTypeCombox.GetValue()
        iscapture = 1
        if iscaptureTxt == "手动抓拍图片":
            iscapture = 1
        elif iscaptureTxt == "手动抓拍视频":
            iscapture = 2
        elif iscaptureTxt == "历史视频":
            iscapture = 3
        elif iscaptureTxt == "行车碰撞视频":
            iscapture = 4
        elif iscaptureTxt == "停车碰撞视频":
            iscapture = 5
        elif iscaptureTxt == "一键报警视频":
            iscapture = 6

        imei = globalParams.homeArea.getDevCodeText().GetValue()
        lat = globalParams.eventArea.getLatTextCtr().GetValue()
        lng = globalParams.eventArea.getLngTextCtr().GetValue()
        timeCur = int(time.time())
        fsize = os.path.getsize(filePath)
        fsize = int(fsize / 1024)
        with open(filePath, "rb") as fi:
            files = {'file': fi}
            params["fileaddress"] = filePath
            params["fsize"] = fsize
            params["cam"] = cam
            params["imei"] = imei
            params["iscapture"] = iscapture
            params["lat"] = lat
            params["lng"] = lng
            params["time"] = timeCur
            res = requests.post("http://api-test.vandyo.com/mirror/upload/file", files=files, params=params)
        curTime = self.getCurTimeTxt()
        wx.CallAfter(pub.sendMessage, "showLog", msg=curTime + "视频上传完成！")

    ####################################################
    #      用于多线程的日志展示
    ####################################################
    def showLog(self,msg):
        self.logTextCtr.WriteText(msg + "\n")



